@@ -28,9 +28,9 @@ def command(
     return cmd(**kwargs)
 
 
-def send(cmd):
+def send(cmd, cwd = os.getcwd()):
     PIPE = subprocess.PIPE
-    return subprocess.Popen(cmd, stdout=PIPE, stderr=PIPE, close_fds=True, shell=True)
+    return subprocess.Popen(cmd, stdout=PIPE, stderr=PIPE, close_fds=True, shell=True, cwd = cwd)
 
 
 def read_output(pipe, funcs):
@@ -41,16 +41,16 @@ def read_output(pipe, funcs):
 
 
 def write_output(get, filepath, show=True):
-    f = open(filepath, 'w', newline='')
-    for line in iter(get, None):
-        line = str(line, 'utf-8', errors='ignore')
-        if show:
-            sys.stdout.write(line)
-        f.write(line)
-    f.close()
+    with open(filepath, 'w', newline='') as f:
+        for line in iter(get, None):
+            line = str(line, 'utf-8', errors='ignore')
+            if show:
+                sys.stdout.write(line)
+            f.write(line)
 
 
-def start_log(process, filepath, show=True):
+
+def start_log(process, filepath, show=True, parallel =False):
     q = queue.Queue()
     out, err = [], []
     tout = threading.Thread(
@@ -61,10 +61,17 @@ def start_log(process, filepath, show=True):
     for t in (tout, terr, twrite):
         t.daemon = False
         t.start()
-    process.wait()
-    for t in (tout, terr):
-        t.join()
-    q.put(None)
+
+
+    if parallel:
+        return q, tout, terr
+    else:
+        process.wait()
+        for t in (tout, terr):
+            t.join()
+        q.put(None)
+        return q, tout, terr
+
 
 
 def execute(
@@ -74,6 +81,7 @@ def execute(
         database,
         outputdirectory,
         show_log=True,
+        parallel = False,
         **kwargs,
 ):
     filename = get_filename(inputfile, ext=False)
@@ -88,13 +96,12 @@ def execute(
     # validate configuration of executable path
     if executable == '':
         raise FileNotFoundError(f'Executable path is empty! Check nextnanopy.config')
-    try:
-        os.chdir(wdir)
-    except (OSError, FileNotFoundError):
+
+
+    if (not os.path.isfile(exe)) or (not os.path.isdir(wdir)):
         raise FileNotFoundError(f'Executable path is invalid: {exe}\nCheck nextnanopy.config')
-    
-    process = send(cmd)
-    start_log(process, logfile, show_log)
+    process = send(cmd, cwd = wdir)
+    queue, tout, terr = start_log(process, logfile, show_log, parallel = parallel)
     os.chdir(cwd)
     info = {
         'process': process,
@@ -103,6 +110,9 @@ def execute(
         'logfile': logfile,
         'cmd': cmd,
         'wdir': wdir,
+        'queue': queue,
+        'tout': tout,
+        'terr': terr,
     }
     return info
 
