@@ -1,6 +1,10 @@
 import os,sys
+import threading
+import time
 import warnings
 import itertools
+import queue
+import nextnanopy
 from nextnanopy.utils.formatting import text_to_lines, lines_to_text
 from nextnanopy.utils.mycollections import DictList
 from nextnanopy.utils.misc import savetxt, get_filename, get_folder, get_file_extension, message_decorator, mkdir_even_if_exists, mkdir_if_not_exist
@@ -98,6 +102,7 @@ class InputFileTemplate(object):
         self.content = None
         self.fullpath = fullpath
         self.product = 'not valid'
+        self.__parallel__ = False
         if fullpath is not None:
             self.load(fullpath)
         if configpath is None:
@@ -243,7 +248,7 @@ class InputFileTemplate(object):
         return self.fullpath
 
     @execute_message
-    def execute(self, show_log = True, convergenceCheck = False, **kwargs):
+    def execute(self, show_log = True, convergenceCheck = False, convergence_check_mode = 'pause',**kwargs):
         """
         Execute the input file located at .fullpath
         Individual kwargs can be passed like 'license' or 'database'
@@ -257,7 +262,12 @@ class InputFileTemplate(object):
         convergenceCheck : bool, optional
             if True, check convergence of the simulation
             (default is False)
-
+        convergence_check_mode: string
+            works only for convergenceCheck = True
+            options:
+                'pause': asks user how to proceed if simulation did not converge (default)
+                'terminate': terminate the script if the simulation did not converge
+                'continue': notify a user but continues execution of script
         kwargs may contain:
             exe : str, optional
                 path to executable
@@ -276,13 +286,13 @@ class InputFileTemplate(object):
         cmd_kwargs = dict(self.default_command_args)
         cmd_kwargs.update(kwargs)
         cmd_kwargs['inputfile'] = self.fullpath
-        info = cmd_execute(show_log=show_log, **cmd_kwargs)
+        info = cmd_execute(show_log=show_log, parallel = self.__parallel__, **cmd_kwargs)
         self.execute_info = info
-        if convergenceCheck:
-            self.check_convergence()
+        if convergenceCheck and not self.__parallel__: #- possible solution for later (delete comment if not)
+            self.check_convergence(mode= convergence_check_mode)
         return info
 
-    def check_convergence(self): 
+    def check_convergence(self, mode = 'pause'):
         if self.product == 'nextnano.MSB':
             raise NotImplementedError('Convergence check has not yet implemented for nextnano.MSB!')
             
@@ -315,21 +325,31 @@ class InputFileTemplate(object):
                             raise RuntimeError(f'\nSimulation got terminated! Check the log:\n{log}')
         except FileNotFoundError:
             print(f'Log file {log} not found!')
-            raise
+            if mode != 'continue':
+                raise
+            else:
+                pass
         except RuntimeError as e:
             print(e)
-            pause = True
-            while pause:
-                answer = input('Do you nevertheless want to continue? [y/n]: ')
-                if answer == 'y' or answer == 'yes': 
-                    pause = False
-                    return
-                elif answer == 'n' or answer == 'no': 
-                    pause = False
-                    raise RuntimeError('Nextnanopy terminated.')
-                else: 
-                    print('Invalid input.')
-                    continue
+            if mode == 'pause':
+                pause = True
+                while pause:
+                    answer = input('Do you nevertheless want to continue? [y/n]: ')
+                    if answer == 'y' or answer == 'yes':
+                        pause = False
+                        return
+                    elif answer == 'n' or answer == 'no':
+                        pause = False
+                        raise RuntimeError('Nextnanopy terminated.')
+                    else:
+                        print('Invalid input.')
+                        continue
+            elif mode == 'terminate':
+                raise RuntimeError('Nextnanopy terminated.')
+            elif mode == 'continue':
+                return
+            else:
+                raise ValueError(f'Mode "{mode}" is not valid')
         else:
             return
 
@@ -472,6 +492,155 @@ class InputFile(InputFileTemplate):
         file.raw_lines = self.raw_lines
         return file.lines
 
+class ExecutionQueue(threading.Thread):
+    """
+        This class take InputFiles and add them in the execution queue.
+        Depending on limit_parallel, InputFiles are executed in parallel or sequentially.
+
+
+        Parameters
+        ----------
+        limit_parallel: int
+            number of InputFiles to be executed in parallel (default: 1)
+        terminate_empty : bool
+            If True, terminates once all added files are executed and logged.
+            If you want to add more input files even after execution of all added in the beginning, use termanate_empy = False
+            Then the ExecutionQueue has to be stopped manually later (ExecutionQueue.stop())
+        convergenceCheck: bool
+            see convergenceCheck in InputFile
+
+        **execution_kwargs: parameters to be taken by InputFile.execute()
+
+
+
+
+        Attributes
+        ----------
+        waiting_queue: queue.Queue
+            queue of InputFile objects to be executed
+        started: list
+            list of (simulation_info:dict, InputFile) currently executing
+
+        finished: list
+            list of simulation_infos for finished simulations
+
+        stop_when_empty: bool
+            see terminate_empty parameter
+        daemon: bool
+            see threading.Thread.daemon
+
+        Methods
+        ----------- for user
+        add(*input_files)
+            adds InputFiles to queue
+
+        start()
+            start the thread (i.e. execution)
+            see threading.Thread.start()
+
+        stop()
+            stop the thread (once all added files are executed)
+            only necessary if termanate_empty = True
+
+
+        -------internal (or for advanced users)
+        all_done()
+            return True if all execution and logging are finished
+
+        add_execution()
+            pop an InputFile from self.waiting_queue, execute and add to self.started
+
+        log_finished()
+            finish logging for finished execution in self.started
+
+        run()
+            commands to be run upon start():
+                pop simulation from queue and execute
+                log the simulation if some are finished from self.started
+            see threading.Thread.run()
+
+    """
+    def __init__(self, limit_parallel : int = 1 , maxsize : int = 0, terminate_empty : bool = True, convergenceCheck = False, **execution_kwargs):
+        super(ExecutionQueue, self).__init__()
+        self.waiting_queue = queue.Queue()#should be queue of InputFile objects
+        self.started = []#should be list of execution_infos
+        self.finished = []#should be list of execution_infos
+        self.limit_parallel = limit_parallel
+        self.execution_kwargs = execution_kwargs
+        self.convergenceCheck = convergenceCheck
+        self.stop_when_empty = terminate_empty
+        self.daemon = False
+    def all_done(self):
+        return not bool(not self.waiting_queue.empty() or self.started)
+
+    def add(self,*input_files: InputFileTemplate):
+        for input_file in input_files:
+            self.waiting_queue.put(input_file)
+
+    def stop(self):
+        self.stop_when_empty = True
+
+
+    def add_execution(self):
+        if (len(self.started) < self.limit_parallel) and not self.waiting_queue.empty():
+            input_f = self.waiting_queue.get()
+            if self.limit_parallel>1:
+                input_f.__parallel__ = True
+            if 'show_log' in self.execution_kwargs and not self.execution_kwargs['show_log']:
+                print(f"\nRemaining simulations in the queue: ", self.waiting_queue.qsize())
+            info = input_f.execute(**self.execution_kwargs)
+            self.started.append((info,input_f))
+
+    def log_finished(self):
+        if self.limit_parallel>1:
+            i = 0
+            while i < len(self.started):
+                poll = self.started[i][0]['process'].poll()
+                if poll is None:
+                    i+=1
+                else:
+                    #TODO check once again del
+                    self.started[i][0]['process'].wait()
+                    tout = self.started[i][0]['tout']
+                    terr = self.started[i][0]['terr']
+                    for t in (tout, terr):
+                        t.join()
+                    self.started[i][0]['queue'].put(None)
+                    if self.convergenceCheck:
+                        if 'convergence_check_mode' in self.execution_kwargs:
+                            convergence_check_mode = self.execution_kwargs['convergence_check_mode']
+                        else:
+                            convergence_check_mode = 'pause'
+                        self.started[i][1].check_convergence(mode = convergence_check_mode)
+                    self.finished.append(self.started[i][0])
+                    del self.started[i]
+
+
+        else:
+            i = 0
+            while i < len(self.started):
+                poll = self.started[i][0]['process'].poll()
+                if poll is None:
+                    i+=1
+                else:
+                    self.finished.append(self.started[i][0])
+                    del self.started[i]
+
+    def run(self):
+        while True:
+            self.add_execution()
+            self.log_finished()
+            if self.all_done():
+                print('\nWaiting queue is empty, all execution and logging are finished')
+
+                if self.stop_when_empty:
+                    break
+                while self.all_done() and not self.stop_when_empty:
+                    time.sleep(0.1) #to ensure switch to main thread
+                    pass
+
+
+
 class Sweep(InputFile):
     """
         This class give a user possibility to run multiple simulations (sweep) over defined variables in the input file.
@@ -548,7 +717,7 @@ class Sweep(InputFile):
             self.input_files.append(inputfile)
 
 
-    def execute_sweep(self, delete_input_files = False, overwrite = False, show_log = True, convergenceCheck = False):
+    def execute_sweep(self, delete_input_files = False, overwrite = False, show_log = True, convergenceCheck = False, convergence_check_mode = 'pause', parallel_limit = 1, **kwargs):
         """
         Execute created input files and saves information to output folder.
 
@@ -568,17 +737,43 @@ class Sweep(InputFile):
             If it did not converge, nextnanopy warns you and ask if you want to proceed with postprocessing. 
             Note that non-converged solutions are not reliable and further calculation and/or visualization from them do not make much sense. 
             Default is False.
+        parallel_limit: int, optional
+            number of simulation to run simultaniously. Espicially usefull for simple simulations which migh be more efficiently rn in parallel. Be aware that
+            some nextnano solvers parallelize computations internally in threads (controlled by --threads in nextnanopy config). To avoid unexpected behaviour and
+            not desirable decrease of simulation speed use the rule: parallel_limit*threads<= number of physical cores of the mahcine
+            default 1
+        **kwargs:
+            see **kwargs of InputFile.execute()
         """
         self.prepare_output(overwrite)
         output_directory = self.sweep_output_directory
+        simulations_info = []
+        polls = []
         if not self.input_files:
             warnings.warn('Nothing was executed in sweep! Input files to execute were not created.')
-        for i, inputfile in enumerate(self.input_files):
-            if not show_log:
-                print(f"\nExecuting simulations [{i+1}/{len(self.input_files)}]...")
-            inputfile.execute(outputdirectory = output_directory, show_log = show_log, convergenceCheck = convergenceCheck)
+
+        #TODO: delete if statement (use execution_queue for both cases)
+        if parallel_limit>1:
+            execution_queue = ExecutionQueue(limit_parallel=parallel_limit, terminate_empty=True, outputdirectory = output_directory, show_log = show_log, convergenceCheck = convergenceCheck, convergence_check_mode = convergence_check_mode, **kwargs)
+            execution_queue.add(*self.input_files)
+            execution_queue.start()
+
+            execution_queue.join()
             if delete_input_files:
-                inputfile.remove()
+                for inputfile in self.input_files:
+                    inputfile.remove()
+        else:
+            for i, inputfile in enumerate(self.input_files):
+                if not show_log:
+                    print(f"\nExecuting simulations [{i+1}/{len(self.input_files)}]...")
+                info = inputfile.execute(outputdirectory = output_directory, show_log = show_log, convergenceCheck = convergenceCheck, convergence_check_mode = convergence_check_mode,**kwargs)
+                #simulations_info.append(info)#unnecessary, delete later
+                #polls = [simulation['process'].poll() for simulation in simulations_info]
+                if delete_input_files:
+                    inputfile.remove()
+
+
+
 
     def mk_dir(self,overwrite = False):
         vars = ''
@@ -599,4 +794,8 @@ class Sweep(InputFile):
             file.write("Input file: '{}' \n".format(self.fullpath))
             file.write("Sweep variables: \n")
             for i in self.var_sweep:
-                file.write("{} = {} \n".format(i,self.var_sweep[i]))
\ No newline at end of file
+                file.write("{} = {} \n".format(i,self.var_sweep[i]))
+
+
+
+
