@@ -7,7 +7,7 @@ import warnings
 from nextnanopy.utils.datasets import Variable, Coord
 from nextnanopy.utils.mycollections import DictList
 from nextnanopy.utils.formatting import best_str_to_name_unit
-from nextnanopy.utils.misc import get_filename, message_decorator
+from nextnanopy.utils.misc import get_filename, message_decorator, start_with_choice
 from nextnanopy import defaults
 
 import pyvista as pv
@@ -370,6 +370,9 @@ class DataFileTemplate(Output):
     def get_loader(self):
         pass
 
+    def export(self, filename, format):
+        raise NotImplementedError('Exporters are not implemented yet')
+
 
 class DataFile(DataFileTemplate):
     def __init__(self, fullpath, product=None):
@@ -446,17 +449,15 @@ class DataFile(DataFileTemplate):
             else:
                 if subplots:
                     fig, ax = plt.subplots(len(self.variables))
-                    for i in range(len(self.variables)):
-                        var = self.variables[i]
+                    for i, var in enumerate(self.variables):
                         im = ax[i].pcolormesh(x.value, y.value, var.value, shading='auto', label=f'{var.name}[{var.unit}]')
                         ax[i].set_xlabel(f'{x.name}[{x.unit}]')
                         ax[i].set_ylabel(f'{y.name}[{y.unit}]')
                         ax[i].set_title(f'{var.name}[{var.unit}]')
                         fig.colorbar(im, ax=ax[i])
                 else:
-                    for i in range(len(self.variables)):
+                    for i, var in enumerate(self.variables):
                         fig, ax = plt.subplots()
-                        var = self.variables[i]
                         im = ax.pcolormesh(x.value, y.value, var.value, shading='auto', label=f'{var.name}[{var.unit}]')
                         ax.set_xlabel(f'{x.name}[{x.unit}]')
                         ax.set_ylabel(f'{y.name}[{y.unit}]')
@@ -486,6 +487,7 @@ class AvsAscii(Output):
         self.load_coords()
 
     def load_raw_metadata(self):
+        possible_keys = ['ndim', 'dim', 'nspace', 'veclen','data', 'field', 'label', 'variable', 'coord']
         info = []
         try:
             with open(self.fld, 'r') as f:
@@ -498,7 +500,9 @@ class AvsAscii(Output):
                     except:
                         if line == '':
                             continue
-                        if line[0] != '#':
+                        # if line[0] != '#':
+                        #     info.append(line)
+                        if start_with_choice(line,*possible_keys):
                             info.append(line)
         except UnicodeDecodeError:
             with open(self.fld,'rb') as f:
@@ -511,7 +515,9 @@ class AvsAscii(Output):
                     line = line.strip()        
                     if line == '':
                         continue
-                    if line[0] != '#':
+                    # if line[0] != '#':
+                    #     info.append(line)
+                    if start_with_choice(line, *possible_keys):
                         info.append(line)
         return info
 
